#include "viewer.h"

#include "lightmanager.h"
#include "gmMath/gm.h"
#include "precisetimer.h"

PreciseTimer timer;

CViewer* CViewer::m_pInstance = NULL;

CViewer::CViewer()
{
}

CViewer::~CViewer()
{
}

void CViewer::destroyViewer()
{
  if(m_pInstance)
    delete m_pInstance;
  m_pInstance = NULL;
}

CViewer* CViewer::getViewer()
{
  if(m_pInstance == NULL)
    m_pInstance = new CViewer();

  return m_pInstance;
}

void CViewer::draw()
{
  //set the matrix mode
  glMatrixMode(GL_MODELVIEW);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

  GLfloat red[]     = {0.9f, 0.2f, 0.2f, 1.0f};
  GLfloat blue[]    = {0.2f, 0.2f, 0.9f, 1.0f};
  GLfloat green[]   = {0.2f, 0.9f, 0.2f, 1.0f};
  GLfloat white[]   = {1.0f, 1.0f, 1.0f, 1.0f};
  GLfloat noColor[] = {0.0f, 0.0f, 0.0f, 1.0f};
  GLfloat ambientcolor[] = {0.0f, 0.0f, 0.0f, 1.0f};

  glMaterialfv(GL_FRONT, GL_AMBIENT, ambientcolor);

  glPushMatrix();

  //TODO: Insert a time-dependent camera rotation around the y-axis

  glDisable(GL_LIGHTING);
  glColor3f(0.5,0.5f,0.5f);
  
  //EXAMPLE: code to enable timer dependent rotation
  //float angle = (timer.GetTime() / 50000) % 360;
  //glRotatef(angle, 0, 1, 0);
  //std::cout << angle << std::endl;


  glBegin(GL_QUADS);

    glNormal3d(0.0,1.0,0.0);
    glVertex3d(-5.0,-3.0,-5.0);
    glVertex3d(-5.0,-3.0,5.0);
    glVertex3d(5.0,-3.0,5.0);
    glVertex3d(5.0,-3.0,-5.0);

  glEnd();

  glEnable(GL_LIGHTING);

  //TODO: insert different objects (at least 2) at different positions
  //use e.g. glMaterialfv(GL_FRONT, GL_DIFFUSE, red) to insert different colors

  //TODO: insert time-dependent movements of the objects

  glPopMatrix();
  glutSwapBuffers();
  glutPostRedisplay();
}

void CViewer::initialize(int argc, char **argv)
{
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_STENCIL | GLUT_RGB);
  glutInitWindowSize(800, 600);
  glutCreateWindow(argv[0]);

  //TODO: replace gluLookAt by your own setLookAt function

  gluLookAt(0.0, 6.0, -15.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

  //glClearColor(1.0f, 1.0f, 1.0f, 1.0f);					// white background
  glClearColor(Globals::clearColor[0], Globals::clearColor[1], Globals::clearColor[2], Globals::clearColor[3]);  // white background
  glClearDepth(1000.0f);										// depth buffer setup
  glDepthFunc(GL_LEQUAL);										// the type of depth testing (less or equal)
  glEnable(GL_DEPTH_TEST);									// enable depth testing
  glShadeModel(GL_SMOOTH);									// employ gouraud shading	
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// set perspective calculations to most accurate

  glEnable(GL_LIGHTING);										// enable lighting

  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);			// polygons should be filled

  CLightManager* lightManager = CLightManager::getLightManager();
  int l0 = lightManager->addPointLight(gmVector4(0.0, 20.0, -16, 1.0), gmVector3(0.0, 0.0, 0.0), gmVector3(0.9, 0.9, 0.9), gmVector3(1.0, 1.0, 1.0));

  timer.StartTimer();
}

void CViewer::setLookAt(GLdouble eye_x, GLdouble eye_y, GLdouble eye_z,
                        GLdouble center_x, GLdouble center_y, GLdouble center_z,
                        GLdouble up_x, GLdouble up_y, GLdouble up_z)
{
  //standard eye position is (0,0,0)
  //standard viewing direction is (0,0,-1)
  //standard up vector is (0,1,0)
  //viewing direction and up vector form a basis together with (1,0,0)

  //query glMatrixMode
  GLint matrixMode;
  glGetIntegerv(GL_MATRIX_MODE, &matrixMode);

  //set glMatrixMode to GL_MODELVIEW
  glMatrixMode(GL_MODELVIEW);

  //TODO: implement the lookAt-function

  //1 compute desired viewing direction
  
  //2 compute vector that completes the base, normalize vectors

  //3 guarantee that the base is orthonormal, i.e. correct the up-vector, if necessary
  //step 2 and 3 can be performed in different order, depending on how you orthogonalize the up-vector

  //4 perform rotation and translation in appropriate order

  //restore old matrixMode
  glMatrixMode(matrixMode);
  return;
}


void CViewer::loop()
{
  // specify that should be called in the glut main loop
  glutDisplayFunc(StaticViewer::display);
  glutReshapeFunc(StaticViewer::reshape);

  glutMainLoop();
}

namespace StaticViewer
{
  void display()
  {
    CViewer::getViewer()->draw();
  }

  void reshape(int w, int h)
  {
    // return, if width or hight of the window is zero
    if(w == 0 || h == 0) return;

    GLint matrixMode;
    glGetIntegerv(GL_MATRIX_MODE, &matrixMode);

    // adjust the size of the OpenGL viewport and frustum
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glFrustum(-0.5,0.5,-0.5,0.5,1,30);

    // switch back to modelview matrix stack
    glMatrixMode(matrixMode);
  }
}
